#!/usr/bin/env python
# Copyright 2008-2009 David D Lowe
# released under the GPLv3
"""Please don't confuse metacity module with metacity_ , they are completely
different, hence the underscore"""

from pigment import PigmentType, Pigment
from pigment_more import DirBasedPigment, DirBasedPigmentType
import pigment
import os, sys
import gconf
import gtk
from gettext import gettext as _
if __name__ == "__main__" or 'pychecker' in sys.argv[0]:
    sys.path.append("..")
import ep_exceptions
import hashlib
import subprocess
from glob import glob
import handy


class MetacityType(DirBasedPigmentType):
    humanName = {"en":"Metacity"}
    humanPluralName = {"en":"Metacity"}
    codeName = "metacity"
    abbreviation = "mt"
    directoryName = "metacity"
    canGeneratePreview = True
    
    attachmentTypeHelp = _("The theme file should be the directory that bears the name of theme, and containing a directory named metacity-1, it should not contain any other directories")
        
    isDirectoryShared = True
    specificChildren = ("metacity-1", "index.theme")

    def get_Pigment(self):
        return Metacity

    publicBaseDir = "/usr/share/themes"
    personalBaseDir = "~/.themes"


class Metacity(DirBasedPigment):
    
    
    def __init__(self):
        DirBasedPigment.__init__(self)
        self.type = MetacityType()
 
    def activate(self, progressCallback=None, shell=None):
        if progressCallback is None:
            progressCallback = lambda(a1): False
        if progressCallback(0.0): return
        client = gconf.client_get_default()
        client.set_string("/apps/metacity/general/theme", self.attDirname)
        progressCallback(1.0)

    
    def is_installed(self, installationDirectory=None):
        """Returns whether this pigment is installed
        installationDirectory: the directory in which to search for already 
        installed pigments"""
        if DirBasedPigment.is_installed(self, installationDirectory):
            if os.path.isdir(os.path.join(self.type.publicBaseDir, self.attDirname, "metacity-1")):
                return True
        return False
    
    def is_activated(self):
        """Returns whether this pigment is applied. To check if the effects
        have been activated regardless of Epidermis' pigments, use is_meat_activated"""
        return self.is_meat_activated()
    
    def is_attachment_correct(self, attachmentIndex=None):
        if not DirBasedPigment.is_attachment_correct(self, attachmentIndex):
            return False
        if attachmentIndex is None:
            att = self.attachments[0]
        else:
            att = self.attachments[attachmentIndex]
        if att[-1] == "/": att = att[:-1]
        dir = att.split("/")[-1]
        if dir in ("metacity-1", "gtk-2.0"):
            return False
        if not os.path.isdir(os.path.join(att,"metacity-1")):
            return False
        if self.get_state() != pigment.STATE_INSTALLED:
            for dir in os.listdir(att):
                if os.path.isdir(dir):
                    if dir != "metacity-1":
                        return False
        if len(os.listdir(os.path.join(att, "metacity-1"))) == 0:
            return False
        return True
    
    def is_meat_installed(self):
        """Returns whether the meat of this pigment or similar to it is installed"""
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        try:
            f1 = glob(self.attPath + "/metacity-1/*") # = file(self.attPath, 'rb')
            f2 = glob("/usr/share/themes/%s/metacity-1/*" % self.attDirname) #file(os.path.join("/usr/share/", self.codeName, self.wallpaperFilename), 'rb')
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            for af in ff:
                try:
                    ss = file(af, "rb").read(8096)
                except:
                    return False
            if not ss:
                break
            mm.update(ss)
        return m1 == m2
        
    
    def is_meat_activated(self):
        """Returns whether the meat of this pigment or similar to it is activated"""
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        client = gconf.client_get_default()
        try:
            f1 = glob(self.attPath + "/metacity-1/*")
            f2 = glob("/usr/share/themes/%s/metacity-1/*" % client.get_string("/apps/metacity/general/theme")) 
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            for af in ff:
                try:
                    ss = file(af, "rb").read(8096)
                except:
                    return False
            if not ss:
                break
            mm.update(ss)
        return m1 == m2
        
    def generate_preview(self):
        return self.generate_preview_with_options()
    
    def generate_preview_with_options(self,  gtk2_rcfile=None):
        if len(self.attPath) < 1:
            return None
        dir = os.path.dirname(os.path.abspath(__file__))
        themeScript = os.path.join(dir, "theme_preview.py")
        subprocess.call(["mkdir", "-p", "/tmp/epidermis"])
        subprocess.call(["rm","-f", "/tmp/epidermis/metacity-preview.png"])
        if gtk2_rcfile is None:
            subprocess.check_call(["python", themeScript, "metacity", self.attPath, "/tmp/epidermis/metacity-preview.png"])
        else:
            env = os.environ
            env["GTK2_RC_FILES"] = gtk2_rcfile
            subprocess.check_call(["python", themeScript, "metacity", self.attPath, "/tmp/epidermis/metacity-preview.png"], env=env)
        try:
            pb = gtk.gdk.pixbuf_new_from_file("/tmp/epidermis/metacity-preview.png")
        except:
            import traceback
            traceback.print_exc()
            return None
        if pb is None:
            return None
        else:
            self.preview = pb
            return pb  
    
    def does_activation_require_root(self):
        """Returns whether the activation requires root rights.
        If it does, the pigment object will need a pexpect bash shell as root
        to be passed to some of its methods"""
        return False
    
    def get_display_data(self, index=0):
        """Returns the object of index to be added to a gtk.ListStore, this
        could be a string, a gtk.gdk.Pixbuf or a custom widget, it should
        be rendered using type.create_cell_renderer"""
        return

def main():
    wp = Metacity()
    print wp.information_xml().toxml()

if __name__ == "__main__":
    main()
